@@ -1,5 +1,5 @@
 /* Quinn Diff, Determine which packages need recompiled for any given architecture */
-/* Copyright (C) 1997-2001 James Troup <james@nocrew.org> */
+/* Copyright (C) 1997-2002 James Troup <james@nocrew.org> */
 
 /* This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -60,9 +60,6 @@ determine_packages_architecture (packages_info *packages)
       packages->buffer_pos = 0;
     }
 
-  binary_architecture_match = xmalloc (strlen (packages_architecture) + 2);
-  sprintf (binary_architecture_match, " %s", packages_architecture);
-
 }
 
 void
@@ -72,7 +69,7 @@ process_packages (packages_info *packages)
   want_node *want_data;
   Qlist *want_list;
   char *source_version;
-  char *old_version;
+  packages_ht_info *old_package;
   int i;
 
   want_list = NULL;
@@ -104,13 +101,15 @@ process_packages (packages_info *packages)
 
       if (packages->architecture)
 	{
-	  if (strncmp(packages_architecture, packages->architecture, strlen(packages_architecture)) &&
-	      strncmp("all", packages->architecture, 3))
+	  if (strcmp(packages_architecture, packages->architecture) &&
+	      strncmp("all", packages->architecture, 4))
 	    {
-	      debug (debug_packages, "parse_packages: [emily] \"%s\" is neither arch: %s or arch: all, skipping.", packages->name, packages_architecture);
+	      debug (debug_packages, "parse_packages: [emily] \"%s\" is neither arch: %s or arch: all, skipping. [%s]", packages->name, packages_architecture, packages->architecture);
 	      continue;
 	    }
 	}
+      else
+	fubar(NONSYS, "parse_packages: %s has no Architecture field.", packages->name);
 
       if (packages->source)
 	{
@@ -127,19 +126,21 @@ process_packages (packages_info *packages)
 	      strncpy (packages->version, source_version, i);
 	      packages->version[i] = '\0';
 	      debug (debug_packages, "naima: after [%s] %s", packages->name, packages->version);
+	      /* Make packages->source be just the source package name */
+	      *(source_version - 2) = '\0';
 	    }
 	}
 
-      old_version = packages_ht_lookup (packages->name);
-      if (old_version)
+      old_package = packages_ht_lookup (packages->name);
+      if (old_package)
 	{ /* Handle duplicates */
-	  i = compare_versions (packages->version, old_version, packages->name);
-	  debug (debug_packages, "parse_packages: %s is duplicated [%s vs. %s => %d]", packages->name, packages->version, old_version, i);
+	  i = compare_versions (packages->version, old_package->version, packages->name);
+	  debug (debug_packages, "parse_packages: %s is duplicated [%s vs. %s => %d]", packages->name, packages->version, old_package->version, i);
 	  if (i < 0)
-	    packages_ht_add (packages->name, packages->version);
+	    packages_ht_add (packages->name, packages->version, packages->source, packages->architecture);
 	}
       else
-	packages_ht_add (packages->name, packages->version);
+	packages_ht_add (packages->name, packages->version, packages->source, packages->architecture);
 
       xfree (packages->version);
       xfree_if_non_null (packages->source);
@@ -163,29 +164,36 @@ packages_ht_create(void)
 }
 
 void
-packages_ht_add (char *package, char *version)
+packages_ht_add (const char *package, const char *version, const char *source, const char *architecture)
 {
 
-  char *key, *data;
+  char *key;
+  packages_ht_info *data;
 
-  debug(debug_packages, "Adding \"%s\" [Version %s] to Packages hash table", package, version);
+  debug(debug_packages, "packages_ht_add: %s_%s (%s) to Packages hash table", package, version, source);
 
   key = xstrdup (package);
-  data = xstrdup (version);
+  data = xmalloc (sizeof(packages_ht_info));
+  data->version = xstrdup(version);
+  if (source)
+    data->source = xstrdup(source);
+  else
+    data->source = NULL;
+  data->architecture = xstrdup(architecture);
 
   g_hash_table_insert (packages_hash_table, key, data);
 
 }
 
-char *
+packages_ht_info *
 packages_ht_lookup (const char *package)
 {
 
-  char *binary_version;
+  packages_ht_info *data;
 
-  binary_version = g_hash_table_lookup (packages_hash_table, package);
+  data = g_hash_table_lookup (packages_hash_table, package);
 
-  return (binary_version);
+  return (data);
 
 }
 
@@ -205,6 +213,9 @@ packages_free_ht_entry (gpointer key,
 {
 
   xfree (key);
+  xfree_if_non_null (((packages_ht_info *) data)->source);
+  xfree (((packages_ht_info *) data)->version);
+  xfree (((packages_ht_info *) data)->architecture);
   xfree (data);
 
 }
