@@ -1,5 +1,5 @@
 /* Quinn Diff, Determine which packages need recompiled for any given architecture */
-/* Copyright (C) 1997-2001 James Troup <james@nocrew.org> */
+/* Copyright (C) 1997-2002 James Troup <james@nocrew.org> */
 
 /* This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,8 +37,10 @@ parse_sources (sources_info *source)
 
   want_node *want_data;
   Qlist *want_list, *list_temp;
-  char *binary_version, *binary, *prefixed_source_name;
+  char *binary, *prefixed_source_name;
+  packages_ht_info *binary_data;
   int j,k,l, q;
+  boolean is_different;
 
   /* Read the Sources file, comparing to the the hash table of the
      Packages file as we go. */
@@ -83,7 +85,7 @@ parse_sources (sources_info *source)
 	}
 
       /* If we're ignoring 'Architecture: all' packages, check this isn't */
-      if (ignore_arch_all && strstr(source->architecture,"all"))
+      if (ignore_arch_all && in_arch_list(source->architecture,"all"))
 	{
 	  debug (debug_sources, "parse_sources: ignoring 'Architecture: all' source packages, and %s is ('%s').", source->name, source->architecture);
 	  continue;
@@ -101,7 +103,7 @@ parse_sources (sources_info *source)
       if (!((want_node *)(list_temp->data))->found)
 	fubar (NONSYS, "Source package \"%s\" has no version field.", source->name);
 
-      debug(debug_sources, "andrea: priority = %s, section = %s, version = %s, binary = %s",
+      debug(debug_sources, "parse_sources: priority = %s, section = %s, version = %s, binary = %s",
 	    source->priority, source->section, source->version, source->binary);
 
       xfree_if_non_null (source->binaries);
@@ -149,16 +151,44 @@ parse_sources (sources_info *source)
 	  binary = list_temp->data;
 	  if (!is_arch_specific(binary))
 	    {
-	      binary_version = packages_ht_lookup (binary);
-	      if (!binary_version || strcmp (source->version, binary_version))
+	      binary_data = packages_ht_lookup (binary);
+	      is_different = FALSE;
+	      /* If this binary package is not compiled... */
+	      if (!binary_data)
 		{
-		  if (binary_version)
-		    q = compare_versions (source->version, binary_version, source->name);
-		  debug (debug_sources, "lisa: %s is different somehow...", binary);
-		  output_ht_add (source, binary_version, q);
-		  if (!strstr(source->architecture,binary_architecture_match) && !strstr(source->architecture,"any"))
-		    error ("warning: %s has an architecture field of \"%s\" which doesn't include %s.", source->name, source->architecture+1, packages_architecture);
+		  debug (debug_sources, "parse_sources: %s from %s is uncompiled...", binary, source->name);
+		  output_ht_add (source, NULL, 0);
+		  is_different = TRUE;
 		}
+	      /* If it is compiled, make sure it's the same version... */
+	      else
+		{
+		  if (strcmp (source->version, binary_data->version))
+		    {
+		      q = compare_versions (source->version, binary_data->version, source->name);
+		      /* If the binary is newer than the source, ensure it's a binary
+                         from this source package */
+		      if ((q >= 1) &&
+			  (binary_data->source && (strcmp (binary_data->source, source->name))))
+			debug(debug_sources, "parse_sources: %s is from %s, not %s and  is a newer version (%s vs. %s) too.", binary, binary_data->source, source->name, binary_data->version, source->version);
+		      else
+			{
+			  debug (debug_sources, "parse_sources: %s from %s is different version-wise (%d)...", binary, source->name, q);
+			  output_ht_add (source, binary_data->version, q);
+			  is_different = TRUE;
+			}
+		    }
+		  else
+		    {
+		      if (strcmp(binary_data->architecture, "all"))
+			output_ht_add (source, binary_data->version, 0);
+		    }
+		}
+
+	      if (is_different &&
+		  !in_arch_list(source->architecture,packages_architecture) &&
+		  !in_arch_list(source->architecture,"any"))
+		error ("warning: %s has an architecture field of \"%s\" which doesn't include %s.", source->name, source->architecture+1, packages_architecture);
 	    }
 	  list_temp = list_temp->next;
 	}
